@@ -42,7 +42,6 @@
 #include "init.h"
 #include "mailbox.h"
 
-#include <pwd.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -51,6 +50,10 @@
 #include <errno.h>
 #include <sys/wait.h>
 
+#ifdef HAVE_LANGINFO_CODESET
+#include <langinfo.h>
+#endif
+
 void toggle_quadoption (int opt)
 {
   int n = opt/4;
@@ -641,6 +644,40 @@ parse_sort (short *val, const char *s, const struct mapping_t *map, BUFFER *err)
   return 0;
 }
 
+static void mutt_set_default (struct option_t *p)
+{
+  switch (p->type & DT_MASK)
+  {
+    case DT_STR:
+      if (!p->init && *((char **) p->data))
+        p->init = (unsigned long) safe_strdup (* ((char **) p->data));
+      break;
+    case DT_PATH:
+      if (!p->init && *((char **) p->data))
+      {
+	char *cp = safe_strdup (*((char **) p->data));
+	mutt_pretty_mailbox (cp);
+        p->init = (unsigned long) cp;
+      }
+      break;
+    case DT_ADDR:
+      if (!p->init && *((ADDRESS **) p->data))
+      {
+	char tmp[HUGE_STRING];
+	rfc822_write_address (tmp, sizeof (tmp), *((ADDRESS **) p->data));
+	p->init = (unsigned long) safe_strdup (tmp);
+      }
+      break;
+    case DT_RX:
+    {
+      REGEXP *pp = (REGEXP *) p->data;
+      if (!p->init && pp->pattern)
+	p->init = (unsigned long) safe_strdup (pp->pattern);
+      break;
+    }
+  }
+}
+
 static void mutt_restore_default (struct option_t *p)
 {
   switch (p->type & DT_MASK)
@@ -691,6 +728,7 @@ static void mutt_restore_default (struct option_t *p)
 	  regfree (pp->rx);
 	  FREE (&pp->rx);
 	}
+
 	if (p->init)
 	{
 	  char *s = (char *) p->init;
@@ -718,6 +756,7 @@ static void mutt_restore_default (struct option_t *p)
       }
       break;
   }
+
   if (p->flags & R_INDEX)
     set_option (OPTFORCEREDRAWINDEX);
   if (p->flags & R_PAGER)
@@ -883,7 +922,8 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
         else if (DTYPE (MuttVars[idx].type) == DT_STR)
         {
 	  *((char **) MuttVars[idx].data) = safe_strdup (tmp->data);
-	  mutt_set_charset (Charset);
+	  if (mutt_strcmp (MuttVars[idx].option, "charset") == 0)
+	    mutt_set_charset (Charset);
         }
         else
         {
@@ -1669,13 +1709,13 @@ void mutt_init (int skip_sys_rc, LIST *commands)
   /* Get some information about the user */
   if ((pw = getpwuid (getuid ())))
   {
+    char rnbuf[STRING];
+
     Username = safe_strdup (pw->pw_name);
     if (!Homedir)
       Homedir = safe_strdup (pw->pw_dir);
-    if ((p = strchr (pw->pw_gecos, ',')))
-      Realname = mutt_substrdup (pw->pw_gecos, p);
-    else
-      Realname = safe_strdup (pw->pw_gecos);
+
+    Realname = safe_strdup (mutt_gecos_name (rnbuf, sizeof (rnbuf), pw));
     Shell = safe_strdup (pw->pw_shell);
   }
   else 
@@ -1773,12 +1813,25 @@ void mutt_init (int skip_sys_rc, LIST *commands)
     FREE (&token.data);
   }
 
+#ifdef HAVE_LANGINFO_CODESET
+  Charset = safe_strdup (nl_langinfo (CODESET));
+#else
+  Charset = safe_strdup ("iso-8859-1");
+#endif
+
+  mutt_set_charset (Charset);
+  
+  
   /* Set standard defaults */
   for (i = 0; MuttVars[i].option; i++)
+  {
+    mutt_set_default (&MuttVars[i]);
     mutt_restore_default (&MuttVars[i]);
+  }
 
   CurrentMenu = MENU_MAIN;
 
+
 #ifndef LOCALES_HACK
   /* Do we have a locale definition? */
   if (((p = getenv ("LC_ALL")) != NULL && p[0]) ||
