@@ -33,10 +33,12 @@
 #include <stdlib.h>
 #include <sys/wait.h>
 #include <errno.h>
-#include <pwd.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#define MIN(a,b) ((a) < (b) ? (a) : (b))
+#define MAX(a,b) ((a) < (b) ? (b) : (a))
+
 BODY *mutt_new_body (void)
 {
   BODY *p = (BODY *) safe_calloc (1, sizeof (BODY));
@@ -79,6 +81,7 @@ void mutt_adv_mktemp (char *s, size_t l)
   else
   {
     strfcpy (tmp, s, sizeof (tmp));
+    mutt_sanitize_filename (tmp, 1);
     snprintf (s, l, "%s/%s", buf, tmp);
     if (lstat (s, &sb) == -1 && errno == ENOENT)
       return;
@@ -301,93 +304,197 @@ void mutt_expand_link (char *newpath, const char *path, const char *link)
 }
 
 char *mutt_expand_path (char *s, size_t slen)
+{
+  return _mutt_expand_path (s, slen, 0);
+}
+
+char *_mutt_expand_path (char *s, size_t slen, int rx)
 {
   char p[_POSIX_PATH_MAX] = "";
-  char *q = NULL;
-  char *next;
+  char q[_POSIX_PATH_MAX] = "";
+  char tmp[_POSIX_PATH_MAX];
+  char *t;
 
-  if (*s == '~')
+  char *tail = ""; 
+
+  int recurse = 0;
+  
+  do 
   {
-    if (*(s + 1) == '/' || *(s + 1) == 0)
-      snprintf (p, sizeof (p), "%s%s", NONULL(Homedir), s + 1);
-    else
-    {
-      struct passwd *pw;
+    recurse = 0;
 
-      q = strchr (s + 1, '/');
-      if (q)
-	*q = 0;
-      if ((pw = getpwnam (s + 1)))
-	snprintf (p, sizeof (p), "%s/%s", pw->pw_dir, q ? q + 1 : "");
-      else
+    switch (*s)
+    {
+      case '~':
       {
-	/* user not found! */
-	if (q)
-	  *q = '/';
-	return (NULL);
+	if (*(s + 1) == '/' || *(s + 1) == 0)
+	{
+	  strfcpy (p, NONULL(Homedir), sizeof (p));
+	  tail = s + 1;
+	}
+	else
+	{
+	  struct passwd *pw;
+	  if ((t = strchr (s + 1, '/'))) 
+	    *t = 0;
+	  if ((pw = getpwnam (s + 1)))
+	  {
+	    strfcpy (p, pw->pw_dir, sizeof (p));
+	    if (t)
+	      tail = t + 1;
+	    else
+	      tail = "";
+	  }
+	  else
+	  {
+	    /* user not found! */
+	    if (t)
+	      *t = '/';
+	    *p = '\0';
+	    tail = s;
+	  }
+	}
+      }
+      break;
+      
+      case '=':
+      case '+':    
+      {
+#ifdef USE_IMAP
+	/* special case: folder = {host}: don't append slash */
+	if (mx_is_imap (NONULL (Maildir)) && Maildir[strlen (Maildir) - 1] == '}')
+	  strfcpy (p, NONULL (Maildir), sizeof (p));
+	else
+#endif
+	  snprintf (p, sizeof (p), "%s/", NONULL (Maildir));
+	
+	tail = s + 1;
+      }
+      break;
+      
+      /* elm compatibility, @ expands alias to user name */
+    
+      case '@':
+      {
+	HEADER *h;
+	ADDRESS *alias;
+	
+	if ((alias = mutt_lookup_alias (s + 1)))
+	{
+	  h = mutt_new_header();
+	  h->env = mutt_new_envelope();
+	  h->env->from = h->env->to = alias;
+	  mutt_default_save (p, sizeof (p), h);
+	  h->env->from = h->env->to = NULL;
+	  mutt_free_header (&h);
+	  /* Avoid infinite recursion if the resulting folder starts with '@' */
+	  if (*p != '@')
+	    recurse = 1;
+	  
+	  tail = "";
+	}
+      }
+      break;
+      
+      case '>':
+      {
+	strfcpy (p, Inbox, sizeof (p));
+	tail = s + 1;
+      }
+      break;
+      
+      case '<':
+      {
+	strfcpy (p, Outbox, sizeof (p));
+	tail = s + 1;
+      }
+      break;
+      
+      case '!':
+      {
+	if (*(s+1) == '!')
+	{
+	  strfcpy (p, LastFolder, sizeof (p));
+	  tail = s + 2;
+	}
+	else 
+	{
+	  strfcpy (p, Spoolfile, sizeof (p));
+	  tail = s + 1;
+	}
+      }
+      break;
+      
+      case '-':
+      {
+	strfcpy (p, LastFolder, sizeof (p));
+	tail = s + 1;
+      }
+      break;
+      
+      default:
+      {
+	*p = '\0';
+	tail = s;
       }
     }
-  }
-  else if (*s == '=' || *s == '+')
-  {
-#ifdef USE_IMAP
-  /* special case: folder = {host}: don't append slash */
-  if (mx_is_imap (NONULL (Maildir)) && Maildir[strlen (Maildir) - 1] == '}')
-    snprintf (p, sizeof (p), "%s%s", NONULL (Maildir), s + 1);
-  else
-#endif
-    snprintf (p, sizeof (p), "%s/%s", NONULL (Maildir), s + 1);
-  }
-  else if (*s == '@')
-  {
-    /* elm compatibility, @ expands alias to user name */
-    HEADER *h;
-    ADDRESS *alias;
 
-    alias = mutt_lookup_alias (s + 1);
-    if (alias != NULL)
+    if (rx && *p && !recurse)
     {
-      h = mutt_new_header();
-      h->env = mutt_new_envelope();
-      h->env->from = h->env->to = alias;
-      mutt_default_save (p, sizeof (p), h);
-      h->env->from = h->env->to = NULL;
-      mutt_free_header (&h);
-      /* Avoid infinite recursion if the resulting folder starts with '@' */
-      if (*p != '@')
-	mutt_expand_path (p, sizeof (p));
+      mutt_rx_sanitize_string (q, sizeof (q), p);
+      snprintf (tmp, sizeof (tmp), "%s%s", q, tail);
     }
-  }
-  else
-  {
-    next = s + 1;
-    if (*s == '>')
-      q = Inbox;
-    else if (*s == '<')
-      q = Outbox;
-    else if (!mutt_strcmp (s, "!!"))	/* elm compatibility */
-    {
-      q = LastFolder;
-      next = s + 2;
-    }
-    else if (*s == '!')
-      q = Spoolfile;
-    else if (*s == '-')
-      q = LastFolder;
     else
-      return s;
-
-    if (!q)
-      return s;
-    snprintf (p, sizeof (p), "%s%s", q, next);
+      snprintf (tmp, sizeof (tmp), "%s%s", p, tail);
+    
+    strfcpy (s, tmp, slen);
   }
+  while (recurse);
 
-  if (*p)
-    strfcpy (s, p, slen); /* replace the string with the expanded version. */
   return (s);
 }
 
 
+char *mutt_gecos_name (char *dest, size_t destlen, struct passwd *pw)
+{
+  regmatch_t pat_match[1];
+  size_t pwnl;
+  int idx;
+  char *p;
+  
+  if (!pw || !pw->pw_gecos) 
+    return NULL;
+
+  memset (dest, 0, destlen);
+  
+  if (GecosMask.rx)
+  {
+    if (regexec (GecosMask.rx, pw->pw_gecos, 1, pat_match, 0) == 0)
+      strfcpy (dest, pw->pw_gecos + pat_match[0].rm_so, 
+	       MIN (pat_match[0].rm_eo - pat_match[0].rm_so + 1, destlen));
+  }
+  else if ((p = strchr (pw->pw_gecos, ',')))
+    strfcpy (dest, pw->pw_gecos, MIN (destlen, p - pw->pw_gecos + 1));
+  else
+    strfcpy (dest, pw->pw_gecos, destlen);
+
+  pwnl = strlen (pw->pw_name);
+
+  for (idx = 0; dest[idx]; idx++)
+  {
+    if (dest[idx] == '&')
+    {
+      memmove (&dest[idx + pwnl], &dest[idx + 1],
+	       MAX(destlen - idx - pwnl - 1, 0));
+      memcpy (&dest[idx], pw->pw_name, MIN(destlen - idx - 1, pwnl));
+      dest[idx] = toupper (dest[idx]);
+    }
+  }
+      
+  return dest;
+}
+  
+
 char *mutt_get_parameter (const char *s, PARAMETER *p)
 {
   for (; p; p = p->next)
