@@ -512,7 +512,7 @@ void mutt_generate_boundary (PARAMETER **parm)
 }
 
 /* analyze the contents of a file to determine which MIME encoding to use */
-static CONTENT *mutt_get_content_info (const char *fname, BODY *b)
+CONTENT *mutt_get_content_info (const char *fname, BODY *b)
 {
   CONTENT *info;
   FILE *fp;
@@ -533,6 +533,7 @@ static CONTENT *mutt_get_content_info (const char *fname, BODY *b)
     linelen++;
     if (ch == '\n')
     {
+      info->crlf++;
       if (whitespace) info->space = 1;
       if (dot) info->dot = 1;
       if (linelen > info->linemax) info->linemax = linelen;
@@ -542,6 +543,7 @@ static CONTENT *mutt_get_content_info (const char *fname, BODY *b)
     }
     else if (ch == '\r')
     {
+      info->crlf++;
       info->cr = 1;
       if ((ch = fgetc (fp)) == EOF)
       {
@@ -824,26 +826,28 @@ static void transform_to_7bit (BODY *a, FILE *fpin)
   }
 }
 
-static const char *get_text_charset (BODY *b, CONTENT *info)
+static const char *get_text_charset (char *d, size_t dlen, BODY *b, CONTENT *info)
 {
   char send_charset[SHORT_STRING];
   char *chsname;
-
+  char *p;
+  
   chsname = mutt_get_send_charset (send_charset, sizeof (send_charset), b, 1);
   
   /* if charset is unknown assume low bytes are ascii compatible */
 
   if ((chsname == NULL || mutt_strcasecmp (chsname, "us-ascii") == 0)
       && info->hibin)
-    return ("unknown-8bit");
-
-  if (info->hibin || !strcasecmp (chsname, "utf-7"))
-    return (chsname);
-
-  if (info->lobin && !strcasecmp (chsname, "iso-2022-jp"))
-    return (chsname);
+    p = "unknown-8bit";
+  else if (info->hibin || !strcasecmp (chsname, "utf-7"))
+    p = chsname;
+  else if (info->lobin && !strcasecmp (chsname, "iso-2022-jp"))
+    p = chsname;
+  else
+    p = "us-ascii";
   
-  return ("us-ascii");
+  strfcpy (d, p, dlen);
+  return d;
 }
 
 /* determine which Content-Transfer-Encoding to use */
@@ -940,7 +944,8 @@ void mutt_set_body_charset(BODY *b, const char *chs)
 void mutt_update_encoding (BODY *a)
 {
   CONTENT *info;
-
+  char chsbuf[SHORT_STRING];
+  
   if ((info = mutt_get_content_info (a->filename, a)) == NULL)
     return;
 
@@ -948,21 +953,11 @@ void mutt_update_encoding (BODY *a)
   mutt_stamp_attachment(a);
   
   if (a->type == TYPETEXT)
-    mutt_set_body_charset(a, get_text_charset(a, info));
+    mutt_set_body_charset(a, get_text_charset(chsbuf, sizeof (chsbuf), a, info));
 
-#ifdef HAVE_PGP
-  /* save the info in case this message is signed.  we will want to do Q-P
-   * encoding if any lines begin with "From " so the signature won't be munged,
-   * for example.
-   */ 
   safe_free ((void **) &a->content);
   a->content = info;
-  info = NULL;
-#endif
 
-
-
-  safe_free ((void **) &info);
 }
 
 BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
@@ -1052,6 +1047,7 @@ BODY *mutt_make_file_attach (const char *path)
   BODY *att;
   CONTENT *info;
   char buf[SHORT_STRING];
+  char chsbuf[SHORT_STRING];
   int n;
   
   if ((info = mutt_get_content_info (path, NULL)) == NULL)
@@ -1094,19 +1090,9 @@ BODY *mutt_make_file_attach (const char *path)
   mutt_stamp_attachment(att);
 
   if (att->type == TYPETEXT)
-    mutt_set_body_charset(att, get_text_charset(att, info));
-  
-#ifdef HAVE_PGP
-  /*
-   * save the info in case this message is signed.  we will want to do Q-P
-   * encoding if any lines begin with "From " so the signature won't be munged,
-   * for example.
-   */
-  att->content = info;
-  info = NULL;
-#endif
+    mutt_set_body_charset(att, get_text_charset(chsbuf, sizeof (chsbuf), att, info));
 
-  safe_free ((void **) &info);
+  att->content = info;
 
   return (att);
 }
@@ -1828,7 +1814,7 @@ void mutt_unprepare_envelope (ENVELOPE *env)
   LIST *item;
 
   for (item = env->userhdrs; item; item = item->next)
-    rfc2047_decode (item->data, item->data,  mutt_strlen (item->data) + 1);
+    rfc2047_decode (&item->data);
 
   rfc822_free_address (&env->mail_followup_to);
 
@@ -1837,7 +1823,7 @@ void mutt_unprepare_envelope (ENVELOPE *env)
   rfc2047_decode_adrlist (env->cc);
   rfc2047_decode_adrlist (env->from);
   rfc2047_decode_adrlist (env->reply_to);
-  rfc2047_decode (env->subject, env->subject, mutt_strlen (env->subject) + 1);
+  rfc2047_decode (&env->subject);
 }
 
 static void _mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to, const char *resent_from)
